/**************************************************************************************

        MUSTUXLIB - THE COMMON LIBRARY FOR ALL MUSTUX APPLICATIONS
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#ifndef MUSTUXDEBUGGER_HH
#define MUSTUXDEBUGGER_HH

#include <stdio.h>

//Debugging Macros

#define CHANGE_COLOR_BLACK   printf("%c[0;31;40m",27)
#define CHANGE_COLOR_RED     printf("%c[0;31;40m",27)
#define CHANGE_COLOR_GREEN   printf("%c[0;32;40m",27)
#define CHANGE_COLOR_ORANGE  printf("%c[0;33;40m",27)
#define CHANGE_COLOR_BLUE    printf("%c[0;34;40m",27)
#define CHANGE_COLOR_MAGENTA printf("%c[0;35;40m",27)
#define CHANGE_COLOR_CYAN    printf("%c[0;36;40m",27)
#define CHANGE_COLOR_WHITE   printf("%c[0;37;40m",27)
#define CHANGE_COLOR_YELLOW  printf("%c[0;38;40m",27)

#define PENTER if (MustuxDebugger::is_debug_on()) \
                                { \
                                MustuxDebugger::fill_tabs(); \
                                CHANGE_COLOR_GREEN; printf("ENTERING "); CHANGE_COLOR_WHITE; printf("%s",__FILE__); printf("::"); printf("%s",__FUNCTION__); printf("\n"); \
                                MustuxDebugger::more_tabs(); \
                                }


#define PEXIT  if (MustuxDebugger::is_debug_on()) \
                                { \
                                MustuxDebugger::less_tabs(); \
                                MustuxDebugger::fill_tabs(); \
                                CHANGE_COLOR_BLUE; printf("LEAVING "); CHANGE_COLOR_WHITE; printf("%s",__FILE__); printf("::"); printf("%s",__FUNCTION__); printf("\n"); \
                                }


#define PENTERCONS if (MustuxDebugger::is_debug_on()) \
                                { \
                                MustuxDebugger::fill_tabs(); \
                                CHANGE_COLOR_GREEN; printf("ENTERING "); CHANGE_COLOR_WHITE; printf("%s",__FUNCTION__); CHANGE_COLOR_CYAN; printf(" (CONSTRUCTOR)"); CHANGE_COLOR_WHITE; printf("\n"); \
                                MustuxDebugger::more_tabs();\
                                }


#define PEXITCONS  if (MustuxDebugger::is_debug_on()) \
                                { \
                                MustuxDebugger::less_tabs(); \
                                MustuxDebugger::fill_tabs(); \
                                CHANGE_COLOR_BLUE; printf("LEAVING "); CHANGE_COLOR_WHITE; printf("%s",__FUNCTION__); CHANGE_COLOR_CYAN; printf(" (CONSTRUCTOR)"); CHANGE_COLOR_WHITE; printf("\n"); \
                                }


#define PENTERDES if (MustuxDebugger::is_debug_on()) \
                                { \
                                MustuxDebugger::fill_tabs(); \
                                CHANGE_COLOR_GREEN; printf("ENTERING "); CHANGE_COLOR_WHITE; printf("%s",__FUNCTION__); CHANGE_COLOR_CYAN; printf(" (DESTRUCTOR)"); CHANGE_COLOR_WHITE; printf("\n"); \
                                MustuxDebugger::more_tabs();\
                                }


#define PEXITDES  if (MustuxDebugger::is_debug_on()) \
                                { \
                                MustuxDebugger::less_tabs(); \
                                MustuxDebugger::fill_tabs(); \
                                CHANGE_COLOR_BLUE; printf("LEAVING "); CHANGE_COLOR_WHITE; printf("%s",__FUNCTION__); CHANGE_COLOR_CYAN; printf(" (DESTRUCTOR)"); CHANGE_COLOR_WHITE; printf("\n"); \
                                }

#define PDEBUG(args...) CHANGE_COLOR_GREEN; printf("DEBUG : "); printf("%s",__FILE__); printf("::");printf("%s",__FUNCTION__); printf(":"); printf(args); CHANGE_COLOR_WHITE; printf("\n");

#define PMARK(x) CHANGE_COLOR_YELLOW; printf("--> Reached mark %s",x); CHANGE_COLOR_WHITE; printf("\n");

#define PERROR(args...) printf("\n"); CHANGE_COLOR_RED; printf("*** Error in "); printf("%s",__PRETTY_FUNCTION__); printf("\n"); printf(args); CHANGE_COLOR_WHITE; printf("\n");

#define PMESG(args...)  if (MustuxDebugger::is_debug_on()) \
                                 { \
                                 MustuxDebugger::fill_tabs(); \
                                 CHANGE_COLOR_MAGENTA; printf("[ "); printf(args); printf(" ]"); CHANGE_COLOR_WHITE; printf("\n");\
                                 }\


#define PWARN(args...)  CHANGE_COLOR_YELLOW; printf("WARNING: "); printf(args); CHANGE_COLOR_WHITE; printf("\n");


/** MustuxDebugger is a collection of macros that makes easier the job of debugging
 *  a Mustux Application.
 *  The list of macros is :
 *  PENTER - Outputs a message when entering a method. Used in the FIRST line of a method;
 *  PEXIT  - Outputs a message when leaving a method. Used in the LAST line of a method,
 *           except if the last line is a return statement (in this case is put immediately before
 *           the return statement
 *  PMESG(message) - Outputs a message
 *  PENTERCONS - Outputs a message when entering a constructor. Similar to PENTER
 *  PEXITCONS  - Outputs a message when leaving a constructor. Similar to PEXIT
 *  PENTERDES  - Outputs a message when entering a destructor. Similar to PENTER
 *  PEXITDES   - Outputs a message when leaving a destructor. Similar to PEXIT
 *  PFLOODENTER - Similar to PENTER but recommended for methods intensively called
 *  PFLOODEXIT  - Similar to PEXIT but recommended for methods intensively called
 */

namespace MustuxDebugger
        {
        /* Used internally by MustuxDebugger. Align the output with the level of execution in a given
        * moment
        */
        void fill_tabs();

        /* Used internally by MustuxDebugger. Increase one level of execution in output messages
        * moment
        */
        void more_tabs();

        /* Used internally by MustuxDebugger. Decrease one level of execution in output messages
        */
        void less_tabs();

        void set_debug_on();
        void set_debug_off();

        /* Used internally by MustuxDebugger. Returns true if debugOn flag is true.
        */
        bool is_debug_on();

        }



#endif



