# Configure paths for Mustux
# Martin Herren		2003-01-02
# 	adapted from
# Configure paths for GTS
# Stphane Popinet  2001-10-4
#       adapted from
# Configure paths for GLIB
# Owen Taylor       97-11-3

dnl
dnl AC_PATH_MUSTUX([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl Test for MUSTUX, and define MUSTUX_CXXFLAGS and MUSTUX_LIBS
dnl
dnl This macro sets the $MUSTUX_CXXFLAGS and $MUSTUX_LIBS variables
dnl 
dnl For this macro to work, a 'Qt detection' macro must have been run before,
dnl setting $QT_CXXFLAGS and $LIBS correctly so that i can make a mustux test-link
dnl
dnl TODO: implement the --disable-mustuxtest option, where no compile and link occurs
dnl
AC_DEFUN(AC_PATH_MUSTUX,
[
  mustux_required_version=ifelse([$1], ,0.16.0,$1)

  AC_LANG(C++)
dnl  BNV_HAVE_QT

  AC_ARG_WITH(mustux-prefix,[  --with-mustux-prefix=PFX   Prefix where Mustux is installed (optional)],
              mustux_config_prefix="$withval", mustux_config_prefix="")
dnl  AC_ARG_ENABLE(mustuxtest, [  --disable-mustuxtest       Do not try to compile and run a test Mustux program],
dnl                , enable_mustuxtest=yes)

  if test x$mustux_config_prefix != x ; then
     mustux_config_args="$mustux_config_args --prefix=$mustux_config_prefix"
     if test x${MUSTUX_CONFIG+set} != xset ; then
        MUSTUX_CONFIG=$mustux_config_prefix/bin/mustux-config
     fi
  fi

  AC_PATH_PROG(MUSTUX_CONFIG, mustux-config, no)

  if test "$MUSTUX_CONFIG" != "no" ; then
    MUSTUX_CXXFLAGS=`$MUSTUX_CONFIG $mustux_config_args --cflags`
    MUSTUX_LIBS=`$MUSTUX_CONFIG $mustux_config_args --libs`
    has_mustux="yes"

    ac_save_CXXFLAGS="$CXXFLAGS"
    ac_save_LIBS="$LIBS"
    CXXFLAGS="$CXXFLAGS $MUSTUX_CXXFLAGS $QT_CXXFLAGS -DQT_THREAD_SUPPORT  -D_REENTRANT"
    LIBS="$LIBS $MUSTUX_LIBS $QT_LIBS"

    AC_CHECK_HEADERS([mustux.h],,has_mustux="no",[#include <qapplication.h>])
    AC_MSG_CHECKING([for libmustux])
    AC_LINK_IFELSE([
#include <mustux.h>

int main(int argc, char **argv)
{
  MustuxApplication *app = new MustuxApplication(argc, argv);
  delete app;

  return 0;
}
],, has_mustux="no")

    if test $has_mustux = "yes"; then
      AC_MSG_RESULT(yes)
    else
      AC_MSG_RESULT(no)
    fi
      

    CXXFLAGS=$ac_save_CXXFLAGS
    LIBS=$ac_save_LIBS

    AC_MSG_CHECKING(for libmustux version >= $mustux_required_version)
    mustux_installed_version=`$MUSTUX_CONFIG --version`
    AC_COMPARE_VERSIONS($mustux_installed_version,$mustux_required_version, \
                        mustux_has_good_version="yes", \
                        mustux_has_good_version="no")

    if test $mustux_has_good_version = "yes"; then
      AC_MSG_RESULT(yes)
    else
      AC_MSG_RESULT(no)
    fi

    AC_SUBST(MUSTUX_CXXFLAGS)
    AC_SUBST(MUSTUX_LIBS)

    if test $mustux_has_good_version = "yes"; then
      if test $has_mustux = "yes"; then
        ifelse([$2], , :,[$2])
      else
        ifelse([$3], , :,[$3])
      fi
    else
      ifelse([$3], , :,[$3])
    fi
  else
    dnl mustux-config not found
    ifelse([$3], , :,[$3])
  fi
]) dnl AC_DEFUN(AM_PATH_MUSTUX)






dnl
dnl AC_COMPARE_VERSIONS(INSTALLED_VERSION,REQUIRED_VERSION,[ACTION-IF-FOUND[, ACTION_IF_NOT_FOUND]])
dnl where versions are strings of type 1.2.10 ore 0.16.0pre4
dnl
AC_DEFUN(AC_COMPARE_VERSIONS,[

  installed_version=$1
  required_version=$2

  installed_major_version=`echo $installed_version | cut -d . -f 1`
  installed_minor_version=`echo $installed_version | cut -d . -f 2`
  installed_micro_version=`echo $installed_version | sed s/pre/./ | cut -d . -f 3`
  installed_prerelease_version=`echo $installed_version | sed s/pre/./ | cut -d . -f 4`
  required_major_version=`echo $required_version | cut -d . -f 1`
  required_minor_version=`echo $required_version | cut -d . -f 2`
  required_micro_version=`echo $required_version | sed s/pre/./ | cut -d . -f 3`
  required_prerelease_version=`echo $required_version | sed s/pre/./ | cut -d . -f 4`
  if (test $installed_major_version -gt $required_major_version); then
    has_good_version="yes"
  else
    if (test $installed_major_version -eq $required_major_version -a \
             $installed_minor_version -gt $required_minor_version); then
      has_good_version="yes"
    else
      if (test $installed_major_version -eq $required_major_version -a \
               $installed_minor_version -eq $required_minor_version -a \
               $installed_micro_version -gt $required_micro_version); then
        has_good_version="yes"
      else
        if (test $installed_major_version -eq $required_major_version -a \
                 $installed_minor_version -eq $required_minor_version -a \
                 $installed_micro_version -eq $required_micro_version); then
          if (test x$installed_prerelease_version = x); then
            has_good_version="yes"
          else
            if (test x$required_prerelease_version = x); then
	      has_good_version="no"
            else
              if (test $installed_prerelease_version -ge $required_prerelease_version); then
                has_good_version="yes"
              else
                has_good_version="no"
              fi
            fi
          fi
        else
          has_good_version="no"
        fi
      fi
    fi
  fi

  if test $has_good_version = "yes"; then
    ifelse([$3], , :,[$3])
  else
    ifelse([$4], , :,[$4])
  fi

]) dnl AC_COMPARE_VERSIONS







dnl AM_PATH_QT([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND [,LIBRARY]]]])
dnl Test for QT, and define QT_CXXFLAGS and QT_LIBS
dnl
AC_DEFUN(AC_PATH_QT,[

  min_qt_version=ifelse([$1], ,3.0.0,$1)

  AC_LANG(C++)

  AC_ARG_WITH(qt-prefix,[  --with-qt-prefix=PFX   Prefix where Qt is installed (optional)],
              qt_config_prefix="$withval", qt_config_prefix="")
  AC_ARG_ENABLE(qttest, [  --qt-mustuxtest       Do not try to compile and run a test Qt program],
                , enable_qttest=yes)

  AC_PATH_PROG(MOC, moc, no)

  if test "$MOC" != "no" ; then

    ac_save_CXXFLAGS="$CXXFLAGS"
    ac_save_LIBS="$LIBS"
dnl    CXXFLAGS="$CXXFLAGS $MUSTUX_CXXFLAGS $QT_CXXFLAGS -DQT_THREAD_SUPPORT  -D_REENTRANT"
dnl    LIBS="$LIBS $MUSTUX_LIBS $QT_LIBS"

    AC_CHECK_HEADERS([qt/qapplication.h])
    AC_MSG_CHECKING([for libqt])
    AC_LINK_IFELSE([
#include <qapplication.h>

int main(int argc, char **argv)
{
  QApplication *app = new QApplication(argc, argv);
  delete app;

  return 0;
}
], AC_MSG_RESULT([yes]), AC_MSG_RESULT([no]))

    CXXFLAGS=$ac_save_CXXFLAGS
    LIBS=$ac_save_LIBS

    AC_MSG_CHECKING(for libmustux version >= $min_mustux_version)

    AC_SUBST(QT_CXXFLAGS)
    AC_SUBST(QT_LIBS)

dnl    if test $mustux_has_good_version = "yes"; then
dnl      if test $has_mustux = "yes"; then
dnl        AC_MSG_RESULT(yes)
dnl        ifelse([$2], , :,[$2])
dnl      else
dnl        ifelse([$3], , :,[$3])
dnl      fi
dnl    else
dnl      AC_MSG_RESULT(no)
dnl      ifelse([$3], , :,[$3])
dnl    fi
dnl  else
dnl    dnl mustux-config not found
dnl    ifelse([$3], , :,[$3])
  fi
])